@@ -17,21 +17,21 @@
 
 **********************************************************************************************/
 
-#include "CAbout.h"
-#include "CMainWindow.h"
 #include "canvas/CCanvas.h"
 #include "config.h"
+#include "CAbout.h"
+#include "CMainWindow.h"
 #include "dem/CDemDraw.h"
 #include "dem/CDemList.h"
 #include "gis/CGisDatabase.h"
 #include "gis/CGisWorkspace.h"
-#include "gis/IGisLine.h"
-#include "gis/WptIcons.h"
 #include "gis/db/CSetupWorkspace.h"
+#include "gis/IGisLine.h"
 #include "gis/prj/IGisProject.h"
 #include "gis/rte/router/CRouterBRouter.h"
 #include "gis/rte/router/CRouterRoutino.h"
 #include "gis/search/CGeoSearchConfig.h"
+#include "gis/search/CGeoSearchWeb.h"
 #include "gis/trk/CActivityTrk.h"
 #include "gis/trk/CDetailsTrk.h"
 #include "gis/trk/CKnownExtension.h"
@@ -40,6 +40,8 @@
 #include "helpers/CToolBarConfig.h"
 #include "helpers/CToolBarSetupDialog.h"
 #include "helpers/CWptIconDialog.h"
+#include "helpers/CWptIconManager.h"
+#include "helpers/CWptIconManager.h"
 #include "map/CMapDraw.h"
 #include "map/CMapItem.h"
 #include "map/CMapList.h"
@@ -92,7 +94,8 @@ CMainWindow::CMainWindow()
     setWindowTitle(WHAT_STR);
     dockRealtime->toggleViewAction()->setChecked(false);
 
-    initWptIcons();
+    IGisItem::init();
+    wptIconManager = new CWptIconManager(this);
 
     IUnit::setUnitType((IUnit::type_e)cfg.value("MainWindow/units",IUnit::eTypeMetric).toInt(), this);
     IUnit::setSlopeMode((IUnit::slope_mode_e)cfg.value("Units/slopeMode", IUnit::eSlopeDegrees).toInt());
@@ -121,6 +124,8 @@ CMainWindow::CMainWindow()
     widgetRtWorkspace = new CRtWorkspace(this);
     dockRealtime->setWidget(widgetRtWorkspace);
 
+    geoSearchWeb = new CGeoSearchWeb(this);
+
     // start ---- restore window geometry -----
     cfg.beginGroup("MainWindow");
     if ( cfg.contains("geometry"))
@@ -198,6 +203,7 @@ CMainWindow::CMainWindow()
     connect(actionToggleDocks,           &QAction::triggered,            this,      &CMainWindow::slotToggleDocks);
     connect(actionFullScreen,            &QAction::triggered,            this,      &CMainWindow::slotFullScreen);
     connect(actionStartQMapTool,         &QAction::triggered,            this,      &CMainWindow::slotStartQMapTool);
+    connect(actionRenameView,            &QAction::triggered,            this,      &CMainWindow::slotRenameView);
     connect(tabWidget,                   &QTabWidget::tabCloseRequested, this,      &CMainWindow::slotTabCloseRequest);
     connect(tabWidget,                   &QTabWidget::currentChanged,    this,      &CMainWindow::slotCurrentTabCanvas);
     connect(tabMaps,                     &QTabWidget::currentChanged,    this,      &CMainWindow::slotCurrentTabMaps);
@@ -287,6 +293,7 @@ CMainWindow::CMainWindow()
     for (QDockWidget * const & dock : docks)
     {
         connect(dock, &QDockWidget::visibilityChanged, this, &CMainWindow::slotDockVisibilityChanged);
+        connect(dock, &QDockWidget::topLevelChanged, this, &CMainWindow::slotDockFloating);
     }
 
 
@@ -430,8 +437,6 @@ CMainWindow::CMainWindow()
     }
 
 
-    loadGISData(qlOpts->arguments);
-
     QTimer::singleShot(100, this, SLOT(slotSanityTest()));
 }
 
@@ -874,16 +879,6 @@ void CMainWindow::slotQuickstart()
 
 void CMainWindow::slotAddCanvas()
 {
-    int i, cnt = 0;
-    for(i = 0; i < tabWidget->count(); i++)
-    {
-        CCanvas * canvas = dynamic_cast<CCanvas*>(tabWidget->widget(i));
-        if(nullptr != canvas)
-        {
-            cnt++;
-        }
-    }
-
     CCanvas * view = addView(QString());
     tabWidget->setCurrentWidget(view);
 
@@ -1515,6 +1510,19 @@ void CMainWindow::hideDocks()
     }
 }
 
+void CMainWindow::slotDockFloating(bool floating)
+{
+    if(floating)
+    {
+        QDockWidget* dock = qobject_cast<QDockWidget*>(sender());
+        if(dock != nullptr)
+        {
+            dock->setWindowFlags(dock->windowFlags() & ~Qt::CustomizeWindowHint);
+            dock->show();
+        }
+    }
+}
+
 void CMainWindow::slotDockVisibilityChanged(bool visible)
 {
     if (visible)
@@ -1561,6 +1569,21 @@ void CMainWindow::slotGeoSearchConfigChanged()
     actionGeoSearch->setIcon(geoSearchConfig->getCurrentIcon());
 }
 
+void CMainWindow::slotRenameView()
+{
+    int idx = tabWidget->currentIndex();
+    QString name = tabWidget->tabText(idx);
+    name = QInputDialog::getText(this, tr("Rename View..."), tr("Enter new name for view"), QLineEdit::Normal, name);
+    if(name.isEmpty())
+    {
+        return;
+    }
+    tabWidget->widget(idx)->setObjectName(name);
+    tabWidget->setTabText(idx, name);
+    tabMaps->setTabText(idx, name);
+    tabDem->setTabText(idx, name);
+}
+
 void CMainWindow::displayRegular()
 {
     if (!dockStates.isEmpty())
