@@ -16,20 +16,20 @@
 
 **********************************************************************************************/
 
-#include "CMainWindow.h"
-#include "GeoMath.h"
 #include "canvas/CCanvas.h"
+#include "CMainWindow.h"
 #include "device/IDevice.h"
 #include "gis/CGisDraw.h"
 #include "gis/CGisListWks.h"
 #include "gis/CGisWorkspace.h"
-#include "gis/IGisItem.h"
 #include "gis/db/macros.h"
+#include "gis/IGisItem.h"
 #include "gis/ovl/CGisItemOvlArea.h"
 #include "gis/prj/IGisProject.h"
 #include "gis/rte/CGisItemRte.h"
 #include "gis/trk/CGisItemTrk.h"
 #include "gis/wpt/CGisItemWpt.h"
+#include "GeoMath.h"
 #include "helpers/CDraw.h"
 #include "helpers/CSettings.h"
 #include "units/IUnit.h"
@@ -44,28 +44,8 @@ const QString IGisItem::noKey;
 
 const QString IGisItem::noName = IGisItem::tr("[no name]");
 
-const IGisItem::color_t IGisItem::colorMap[] =
-{
-    {"Black",        QColor(Qt::black),       QString("://icons/8x8/bullet_black.png")}
-    ,{"DarkRed",     QColor(Qt::darkRed),     QString("://icons/8x8/bullet_dark_red.png")}
-    ,{"DarkGreen",   QColor(Qt::darkGreen),   QString("://icons/8x8/bullet_dark_green.png")}
-    ,{"DarkYellow",  QColor(Qt::darkYellow),  QString("://icons/8x8/bullet_dark_yellow.png")}
-    ,{"DarkBlue",    QColor(Qt::darkBlue),    QString("://icons/8x8/bullet_dark_blue.png")}
-    ,{"DarkMagenta", QColor(Qt::darkMagenta), QString("://icons/8x8/bullet_dark_magenta.png")}
-    ,{"DarkCyan",    QColor(Qt::darkCyan),    QString("://icons/8x8/bullet_dark_cyan.png")}
-    ,{"LightGray",   QColor(Qt::lightGray),   QString("://icons/8x8/bullet_gray.png")}
-    ,{"DarkGray",    QColor(Qt::darkGray),    QString("://icons/8x8/bullet_dark_gray.png")}
-    ,{"Red",         QColor(Qt::red),         QString("://icons/8x8/bullet_red.png")}
-    ,{"Green",       QColor(Qt::green),       QString("://icons/8x8/bullet_green.png")}
-    ,{"Yellow",      QColor(Qt::yellow),      QString("://icons/8x8/bullet_yellow.png")}
-    ,{"Blue",        QColor(Qt::blue),        QString("://icons/8x8/bullet_blue.png")}
-    ,{"Magenta",     QColor(Qt::magenta),     QString("://icons/8x8/bullet_magenta.png")}
-    ,{"Cyan",        QColor(Qt::cyan),        QString("://icons/8x8/bullet_cyan.png")}
-    ,{"White",       QColor(Qt::white),       QString("://icons/8x8/bullet_white.png")}
-    ,{"Transparent", QColor(Qt::transparent), QString()}
-};
+QVector<IGisItem::color_t> IGisItem::colorMap;
 
-const size_t IGisItem::colorMapSize = sizeof(colorMap) / sizeof(color_t);
 
 IGisItem::IGisItem(IGisProject *parent, type_e typ, int idx)
     : QTreeWidgetItem(parent, typ)
@@ -165,6 +145,72 @@ IGisItem::~IGisItem()
 {
 }
 
+
+void IGisItem::init()
+{
+    colorMap =
+    {
+        {"Black",       tr("Black"),      QColor(Qt::black),       QString("://icons/8x8/bullet_black.png")}
+        ,{"DarkRed",     tr("Dark Red"),    QColor(Qt::darkRed),     QString("://icons/8x8/bullet_dark_red.png")}
+        ,{"DarkGreen",   tr("Dark Green"),  QColor(Qt::darkGreen),   QString("://icons/8x8/bullet_dark_green.png")}
+        ,{"DarkYellow",  tr("Dark Yellow"), QColor(Qt::darkYellow),  QString("://icons/8x8/bullet_dark_yellow.png")}
+        ,{"DarkBlue",    tr("Dark Blue"),   QColor(Qt::darkBlue),    QString("://icons/8x8/bullet_dark_blue.png")}
+        ,{"DarkMagenta", tr("Dark Magenta"),QColor(Qt::darkMagenta), QString("://icons/8x8/bullet_dark_magenta.png")}
+        ,{"DarkCyan",    tr("Dark Cyan"),   QColor(Qt::darkCyan),    QString("://icons/8x8/bullet_dark_cyan.png")}
+        ,{"LightGray",   tr("Light Gray"),  QColor(Qt::lightGray),   QString("://icons/8x8/bullet_gray.png")}
+        ,{"DarkGray",    tr("Dark Gray"),   QColor(Qt::darkGray),    QString("://icons/8x8/bullet_dark_gray.png")}
+        ,{"Red",         tr("Red"),        QColor(Qt::red),         QString("://icons/8x8/bullet_red.png")}
+        ,{"Green",       tr("Green"),      QColor(Qt::green),       QString("://icons/8x8/bullet_green.png")}
+        ,{"Yellow",      tr("Yellow"),     QColor(Qt::yellow),      QString("://icons/8x8/bullet_yellow.png")}
+        ,{"Blue",        tr("Blue"),       QColor(Qt::blue),        QString("://icons/8x8/bullet_blue.png")}
+        ,{"Magenta",     tr("Magenta"),    QColor(Qt::magenta),     QString("://icons/8x8/bullet_magenta.png")}
+        ,{"Cyan",        tr("Cyan"),       QColor(Qt::cyan),        QString("://icons/8x8/bullet_cyan.png")}
+        ,{"White",       tr("White"),      QColor(Qt::white),       QString("://icons/8x8/bullet_white.png")}
+        ,{"Transparent", tr("Transparent"),QColor(Qt::transparent), QString()}
+    };
+}
+
+qint32 IGisItem::selectColor(QWidget * parent)
+{
+    qint32 colorIdx     = NOIDX;
+    QMenu * menu        = getColorMenu("", nullptr, "", parent);
+    QAction * action    = menu->exec(QCursor::pos());
+
+    if(action != nullptr)
+    {
+        bool ok = false;
+        colorIdx = action->property("colorIdx").toInt(&ok);
+        if(!ok)
+        {
+            colorIdx = NOIDX;
+        }
+    }
+
+    delete menu;
+    return colorIdx;
+}
+
+QMenu * IGisItem::getColorMenu(const QString& title, QObject * obj, const char * slot, QWidget * parent)
+{
+    QMenu * menu = new QMenu(title, parent);
+    menu->setIcon(QIcon("://icons/32x32/SelectColor.png"));
+
+    QAction * action;
+    for(qint32 i = 0; i < IGisItem::colorMap.size(); i++)
+    {
+        QPixmap pixmap(16,16);
+        pixmap.fill(IGisItem::colorMap[i].color);
+        action = menu->addAction(QIcon(pixmap), IGisItem::colorMap[i].label);
+        action->setProperty("colorIdx", i);
+
+        if(obj != nullptr)
+        {
+            QAction::connect(action, SIGNAL(triggered(bool)), obj, slot);
+        }
+    }
+    return menu;
+}
+
 IGisProject * IGisItem::getParentProject() const
 {
     return dynamic_cast<IGisProject*>(parent());
@@ -487,6 +533,30 @@ void IGisItem::cutHistoryBefore()
     }
 }
 
+void IGisItem::squashHistory()
+{
+    if(history.events.size() < 2)
+    {
+        return;
+    }
+
+    history_event_t& first = history.events.first();
+    history_event_t& last = history.events.last();
+
+    last.time    = first.time;
+    last.who     = first.who;
+    last.icon    = first.icon;
+    last.comment = first.comment;
+
+    history.histIdxCurrent = 0;
+    history.histIdxInitial = 0;
+
+    while(history.events.size() > 1)
+    {
+        history.events.pop_front();
+    }
+}
+
 bool IGisItem::isReadOnly() const
 {
     return !(flags & eFlagWriteAllowed) || isOnDevice();
@@ -626,7 +696,7 @@ void IGisItem::showIcon()
 
 QColor IGisItem::str2color(const QString& name)
 {
-    for(size_t i = 0; i < colorMapSize; i++)
+    for(int i = 0; i < colorMap.size(); i++)
     {
         if(QString(colorMap[i].name).toUpper() == name.toUpper())
         {
@@ -639,7 +709,7 @@ QColor IGisItem::str2color(const QString& name)
 
 QString IGisItem::color2str(const QColor& color)
 {
-    for(size_t i = 0; i < colorMapSize; i++)
+    for(int i = 0; i < colorMap.size(); i++)
     {
         if(colorMap[i].color == color)
         {
@@ -737,11 +807,7 @@ QString IGisItem::createText(bool isReadOnly, const QString& cmt, const QString&
     if(!isReadOnly || !isEmpty)
     {
         str += toLink(isReadOnly, "description", tr("<h4>Description:</h4>"), key);
-        if(removeHtml(desc).simplified().isEmpty())
-        {
-            str += tr("<p>--- no description ---</p>");
-        }
-        else
+        if(!removeHtml(desc).simplified().isEmpty())
         {
             str += desc;
         }
@@ -751,11 +817,7 @@ QString IGisItem::createText(bool isReadOnly, const QString& cmt, const QString&
     if(!isReadOnly || !isEmpty)
     {
         str += toLink(isReadOnly, "comment", tr("<h4>Comment:</h4>"), key);
-        if(isEmpty)
-        {
-            str += tr("<p>--- no comment ---</p>");
-        }
-        else
+        if(!isEmpty)
         {
             str += cmt;
         }
@@ -765,11 +827,7 @@ QString IGisItem::createText(bool isReadOnly, const QString& cmt, const QString&
     if(!isReadOnly || !isEmpty)
     {
         str += toLink(isReadOnly, "links", tr("<h4>Links:</h4>"), key);
-        if(isEmpty)
-        {
-            str += tr("<p>--- no links ---</p>");
-        }
-        else
+        if(!isEmpty)
         {
             for(const link_t &link : links)
             {
@@ -947,7 +1005,7 @@ bool IGisItem::getNameAndProject(QString &name, IGisProject *&project, const QSt
         return false;
     }
 
-    project = CGisWorkspace::self().selectProject();
+    project = CGisWorkspace::self().selectProject(false);
     return nullptr != project;
 }
 
