@@ -16,17 +16,18 @@
 
 **********************************************************************************************/
 
-#include "CMainWindow.h"
 #include "canvas/CCanvas.h"
+#include "CMainWindow.h"
 #include "gis/rte/router/brouter/CRouterBRouterSetup.h"
 #include "gis/rte/router/brouter/CRouterBRouterSetupWizard.h"
+#include "helpers/CWebPage.h"
 #include "setup/IAppSetup.h"
 #include <JlCompress.h>
+#include <proj_api.h>
 #include <QFileDialog>
 #include <QMessageBox>
 #include <QNetworkReply>
 #include <QNetworkRequest>
-#include <proj_api.h>
 
 CRouterBRouterSetupWizard::CRouterBRouterSetupWizard()
     : QWizard(CMainWindow::getBestWidgetForParent())
@@ -53,6 +54,10 @@ CRouterBRouterSetupWizard::CRouterBRouterSetupWizard()
 
     connect(pushCreateOrUpdateLocalInstall, &QPushButton::clicked, this, &CRouterBRouterSetupWizard::slotCreateOrUpdateLocalInstallClicked);
 
+    localVersionsPage = new CWebPage(this);
+    webLocalBRouterVersions->setPage(localVersionsPage);
+    connect(localVersionsPage, &CWebPage::linkClicked, this, &CRouterBRouterSetupWizard::slotLocalDownloadLinkClicked);
+    connect(localVersionsPage, &QWebEnginePage::loadFinished, this, &CRouterBRouterSetupWizard::slotWebLocalBRouterVersionsLoadFinished);
     connect(pushLocalInstall, &QPushButton::clicked, this, &CRouterBRouterSetupWizard::slotLocalDownloadButtonClicked);
 
     connect(listProfiles, &QListView::clicked, this, &CRouterBRouterSetupWizard::slotProfileClicked);
@@ -480,20 +485,24 @@ void CRouterBRouterSetupWizard::slotCreateOrUpdateLocalInstallClicked()
 void CRouterBRouterSetupWizard::initLocalInstall()
 {
     pageLocalInstallation->setSetup(setup);
-    connect(webLocalBRouterVersions, &QWebView::loadFinished, this, &CRouterBRouterSetupWizard::slotWebLocalBRouterVersionsLoadFinished);
-    webLocalBRouterVersions->load(QUrl(setup->binariesUrl));
-    QWebPage *localVersionsPage = webLocalBRouterVersions->page();
-    localVersionsPage->setLinkDelegationPolicy(QWebPage::DelegateAllLinks);
-    connect(localVersionsPage, &QWebPage::linkClicked, this, &CRouterBRouterSetupWizard::slotLocalDownloadLinkClicked);
+    localInstallLoaded = false;
+    localVersionsPage->load(QUrl(setup->binariesUrl));
 }
 
 void CRouterBRouterSetupWizard::slotWebLocalBRouterVersionsLoadFinished(bool ok)
 {
-    if (!ok)
+    if (!localInstallLoaded)
     {
-        textLocalInstall->setVisible(true);
-        textLocalInstall->setTextColor(Qt::red);
-        textLocalInstall->append(tr("Error loading installation-page at %1").arg(setup->binariesUrl));
+        if (!ok)
+        {
+            textLocalInstall->setVisible(true);
+            textLocalInstall->setTextColor(Qt::red);
+            textLocalInstall->append(tr("Error loading installation-page at %1").arg(setup->binariesUrl));
+        }
+        else
+        {
+            localInstallLoaded = true;
+        }
     }
 }
 
