@@ -24,14 +24,16 @@
 #include "gis/trk/CActivityTrk.h"
 #include "gis/trk/CTrackData.h"
 #include "gis/trk/filter/CFilterSpeed.h"
+#include "gis/trk/filter/CFilterSpeedCycle.h"
+#include "gis/trk/filter/CFilterSpeedHike.h"
 #include "helpers/CLimit.h"
 #include "helpers/CValue.h"
 
+#include <functional>
+#include <interpolation.h>
 #include <QDebug>
 #include <QPen>
 #include <QPointer>
-#include <functional>
-#include <interpolation.h>
 
 using std::numeric_limits;
 
@@ -48,7 +50,6 @@ class CPropertyTrk;
 class CFitStream;
 class CCanvas;
 
-#define TRK_N_COLORS          17
 #define ASCENT_THRESHOLD       5
 #define MIN_WIDTH_INFO_BOX   300
 
@@ -228,6 +229,7 @@ public:
     const QDateTime& getTimeStart()           const { return timeStart;        }
     const QDateTime& getTimeEnd()             const { return timeEnd;          }
     qint32 getNumberOfVisiblePoints()         const { return cntVisiblePoints; }
+    qint32 getNumberOfInvalidPoints()         const { return cntInvalidPoints; }
     const CActivityTrk& getActivities()       const { return activities;       }
     const CPropertyTrk * getPropertyHandler() const { return propHandler;      }
     const CTrackData::trkpt_t * getMouseMoveFocusPoint()  const { return mouseMoveFocus;   }
@@ -294,7 +296,7 @@ public:
 private:
     void drawColorized(QPainter &p) const;
     void drawColorizedByActivity(QPainter& p) const;
-    void setPen(QPainter& p, QPen& pen, quint32 flag) const;
+    void setPen(QPainter& p, QPen& pen, trkact_t act) const;
     /**@}*/
 
 
@@ -401,7 +403,7 @@ public:
     void combine(const QList<key_t> &keys);
 
     /**
-       @brief Set the CTrackData::trkpt_t::eHidden flag
+       @brief Set the CTrackData::trkpt_t::eFlagHidden flag
 
        The flag is set for all track points between mouseClickFocus and mouseMoveFocus,
        regardless of their previous state.
@@ -410,7 +412,7 @@ public:
     void hideSelectedPoints();
 
     /**
-       @brief Reset the CTrackData::trkpt_t::eHidden flag
+       @brief Reset the CTrackData::trkpt_t::eFlagHidden flag
 
        The flag is reset for all track points between mouseClickFocus and mouseMoveFocus,
        regardless of their previous state.
@@ -422,7 +424,7 @@ public:
        @brief Set the activity flag for all track points
        @param flag  one of CTrackData::trkpt_t::flag_e::eAct...
      */
-    void setActivity(quint32 flags);
+    void setActivity(trkact_t act);
 
     /**
        @brief Sets the activity flag for a selected range of track points
@@ -430,13 +432,13 @@ public:
        The range has to be selected already. The activity will be selected by a dialog displayed
        in this method.
      */
-    void setActivityRange(quint32 flags);
+    void setActivityRange(trkact_t act);
 
     /**
        @brief Copy a section into a new track object
 
        The section is defined by mouseClickFocus and mouseMoveFocus, All points are copied,
-       including the hidden (CTrackData::trkpt_t::eHidden) ones.
+       including the hidden (CTrackData::trkpt_t::eFlagHidden) ones.
 
      */
     void copySelectedPoints() const;
@@ -537,8 +539,10 @@ public:
 
     /** @param speed speed in meter per seconds */
     void filterSpeed(qreal speed);
-    void filterSpeed(const CFilterSpeed::cycling_type_t &cyclingType);
-    void filterGetSlopeLimits(qreal &minSlope, qreal &maxSlope);
+    void filterSpeed(const CFilterSpeedCycle::cycling_type_t &cyclingType);
+    void filterSpeed(const CFilterSpeedHike::hiking_type_t &hikingType);
+
+    void filterGetSlopeLimits(qreal &minSlope, qreal &maxSlope) const;
 
     void filterTerrainSlope();
     void filterReplaceElevation(CCanvas *canvas);
@@ -549,6 +553,8 @@ public:
     void filterDeleteExtension(const QString &ext);
     void filterSubPt2Pt();
     void filterChangeStartPoint(qint32 idxNewStartPoint, const QString &wptName);
+    void filterLoopsCut(qreal dist);
+    void filterZeroSpeedDriftCleaner(qreal distance, qreal ratio);
     /** @} */
 
     /**
@@ -775,8 +781,12 @@ private:
     qreal totalElapsedSeconds = 0;
     qreal totalElapsedSecondsMoving = 0;
     quint32 numberOfAttachedWpt = 0;
+
+    void checkForInvalidPoints();
     /**@}*/
 
+
+
     /**
         \defgroup DrawUtilies Objects used to draw the track
      */
@@ -789,8 +799,8 @@ private:
     QPolygonF lineSimple;   //< the current track line as screen pixel coordinates
     QPolygonF lineFull;     //< visible and invisible points
 
-    qint32 penWidthFg = 3;  //< inner trackline width
-    qint32 penWidthBg = 5;  //< outer trackline width
+    qint32 penWidthFg = 1;  //< inner trackline width
+    qint32 penWidthBg = 3;  //< outer trackline width
     qint32 penWidthHi = 11; //< highlighted trackline width
 
     qint32 widthInfoBox = MIN_WIDTH_INFO_BOX;
@@ -803,7 +813,7 @@ private:
 
     fValueOnChange onChange = [this](const QVariant& val)
                               {
-                                  int w = qRound(3.0 * val.toDouble());
+                                  int w = qRound(1.0 * val.toDouble());
 
                                   penWidthFg = w;
                                   penWidthBg = w + 2;
