@@ -17,21 +17,21 @@
 
 **********************************************************************************************/
 
-#include "CMainWindow.h"
-#include "GeoMath.h"
 #include "canvas/CCanvas.h"
+#include "CMainWindow.h"
 #include "gis/CGisDraw.h"
 #include "gis/CGisListWks.h"
-#include "gis/WptIcons.h"
 #include "gis/prj/IGisProject.h"
 #include "gis/wpt/CDetailsGeoCache.h"
 #include "gis/wpt/CDetailsWpt.h"
 #include "gis/wpt/CGisItemWpt.h"
 #include "gis/wpt/CScrOptWpt.h"
 #include "gis/wpt/CScrOptWptRadius.h"
-#include "gis/wpt/CSetupNewWpt.h"
+#include "gis/wpt/CSetupIconAndName.h"
+#include "GeoMath.h"
 #include "helpers/CDraw.h"
 #include "helpers/CSettings.h"
+#include "helpers/CWptIconManager.h"
 #include "mouse/IMouse.h"
 #include "units/IUnit.h"
 
@@ -200,11 +200,15 @@ void CGisItemWpt::setSymbol()
     setIcon();
 }
 
-bool CGisItemWpt::getNewWptData(QPointF& pt, QString& icon, QString& name)
+QString CGisItemWpt::getLastName(const QString& name)
 {
     SETTINGS;
-    QString lastName = cfg.value("Waypoint/lastName", "wpt").toString();
-    QString lastIcon = cfg.value("Waypoint/lastIcon", "Waypoint").toString();
+    QString lastName = name;
+
+    if(lastName.isEmpty())
+    {
+        lastName = cfg.value("Waypoint/lastName", "wpt").toString();
+    }
 
     const int s = lastName.size();
     if(s != 0)
@@ -228,13 +232,22 @@ bool CGisItemWpt::getNewWptData(QPointF& pt, QString& icon, QString& name)
         }
     }
 
+    cfg.setValue("Waypoint/lastName", lastName);
+    return lastName;
+}
+
+bool CGisItemWpt::getIconAndName(QString& icon, QString& name)
+{
+    SETTINGS;
+    QString lastIcon = cfg.value("Waypoint/lastIcon", "Waypoint").toString();
+
     if(name.isEmpty())
     {
-        name = lastName;
+        name = getLastName("");
     }
     icon = lastIcon;
 
-    CSetupNewWpt dlg(pt, icon, name, CMainWindow::getBestWidgetForParent());
+    CSetupIconAndName dlg(icon, name, CMainWindow::getBestWidgetForParent());
     if(dlg.exec() != QDialog::Accepted)
     {
         return false;
@@ -246,6 +259,29 @@ bool CGisItemWpt::getNewWptData(QPointF& pt, QString& icon, QString& name)
     return true;
 }
 
+void CGisItemWpt::newWpt(QPointF& pt, const QString &name, const QString &desc, IGisProject * project)
+{
+    SETTINGS;
+    QString icon = cfg.value("Waypoint/lastIcon", "Waypoint").toString();
+
+    QString _name = name;
+
+    if(_name.isEmpty())
+    {
+        _name = getLastName("");
+    }
+
+    CGisItemWpt * wpt = new CGisItemWpt(pt, _name, icon, project);
+    if(!desc.isEmpty())
+    {
+        wpt->setDescription(desc);
+    }
+    wpt->editInitial();
+
+    cfg.setValue("Waypoint/lastName", wpt->getName());
+    cfg.setValue("Waypoint/lastIcon", wpt->getIconName());
+}
+
 QString CGisItemWpt::getInfo(quint32 feature) const
 {
     QString str = "<div>";
@@ -334,6 +370,19 @@ QString CGisItemWpt::getInfo(quint32 feature) const
             str += tr("Created: %1").arg(IUnit::datetime2string(wpt.time, false, QPointF(wpt.lon*DEG_TO_RAD, wpt.lat*DEG_TO_RAD)));
         }
     }
+
+    if((feature & eFeatureShowLinks) && !wpt.links.isEmpty())
+    {
+        for(const link_t& link : wpt.links)
+        {
+            if(link.type.isEmpty() || (link.type == "text/html"))
+            {
+                str += "<br/>\n";
+                str += QString("<a href='%1'>%2</a>").arg(link.uri.toString()).arg(link.text);
+            }
+        }
+    }
+
     return str + "</div>";
 }
 
@@ -374,11 +423,11 @@ void CGisItemWpt::setIcon()
 {
     if(geocache.hasData)
     {
-        IGisItem::setIcon(getWptIconByName(geocache.type, focus));
+        IGisItem::setIcon(CWptIconManager::self().getWptIconByName(geocache.type, focus));
     }
     else
     {
-        IGisItem::setIcon(getWptIconByName(wpt.sym, focus));
+        IGisItem::setIcon(CWptIconManager::self().getWptIconByName(wpt.sym, focus));
     }
 }
 
@@ -437,7 +486,7 @@ void CGisItemWpt::setIcon(const QString& name)
 
     QPointF focus;
     QString path;
-    getWptIconByName(name, focus, &path);
+    CWptIconManager::self().getWptIconByName(name, focus, &path);
 
     changed(tr("Changed icon"), path);
 }
@@ -521,6 +570,14 @@ void CGisItemWpt::edit()
     }
 }
 
+void CGisItemWpt::editInitial()
+{
+    CDetailsWpt dlg(*this, CMainWindow::getBestWidgetForParent());
+    dlg.disableHistory();
+    dlg.exec();
+    squashHistory();
+}
+
 void CGisItemWpt::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF> &blockedAreas, CGisDraw *gis)
 {
     posScreen = QPointF(wpt.lon * DEG_TO_RAD, wpt.lat * DEG_TO_RAD);
@@ -586,6 +643,11 @@ void CGisItemWpt::drawItem(QPainter& p, const QRectF& viewport, CGisDraw * gis)
 
 void CGisItemWpt::drawLabel(QPainter& p, const QPolygonF &viewport, QList<QRectF> &blockedAreas, const QFontMetricsF &fm, CGisDraw *gis)
 {
+    if(flags & eFlagWptBubble)
+    {
+        return;
+    }
+
     if(posScreen == NOPOINTF)
     {
         return;
